import 'package:flutter_libserialport/flutter_libserialport.dart';
import 'dart:typed_data';

class MSPCommunication {
  late SerialPort port;

  MSPCommunication(String portName) {
    port = SerialPort(portName);

    if (!port.isOpen) {
      bool opened = port.openReadWrite();
      if (!opened) {
        throw SerialPortError('Failed to open the port.');
      }
    }
  }

  Future<void> sendMessage(int code, String data) async {
    try {
      Uint8List message = encodeMessage(code, data);
      port.write(message);
      await Future.delayed(Duration(seconds: 1));  // simulate some delay
      List<int> response = port.read(64);

      if (response.isNotEmpty) {
        print('Response Data: ${String.fromCharCodes(response)}');
      } else {
        print('No response received from the device.');
      }
    } finally {
      port.close();
    }
  }

  Uint8List encodeMessage(int code, String data) {
    final dataLength = data.length;
    final bufferSize = dataLength + 6;  // Add length for start byte, message type, length, code and checksum
    final buffer = Uint8List(bufferSize);

    buffer[0] = 36; // start byte '$'
    buffer[1] = 77; // message type 'M'
    buffer[2] = 60; // '<'
    buffer[3] = dataLength;
    buffer[4] = code;

    int checksum = buffer[3] ^ buffer[4];

    for (int i = 0; i < dataLength; i++) {
      buffer[5 + i] = data.codeUnitAt(i);
      checksum ^= buffer[5 + i];
    }

    buffer[bufferSize - 1] = checksum;

    return buffer;
  }
}